/*=====================================================================
 * Any authorized distribution of any copy of this code (including any
 * related documentation) must reproduce the following restrictions,
 * disclaimer and copyright notice:
 *
 * The Genesys name, the trademarks and/or logo(s) of Genesys shall
 * not be used to name (even as a part of another name), endorse and/or
 * promote products derived from this code without prior written
 * permission from Genesys Telecommunications Laboratories, Inc.
 *
 * The use, copy, and/or distribution of this code is subject to the
 * terms of the Genesys Developer License Agreement.  This code shall
 * not be used, copied, and/or distributed under any other license
 * agreement.
 *
 * THIS CODE IS PROVIDED BY GENESYS TELECOMMUNICATIONS LABORATORIES,
 * INC. ("GENESYS") "AS IS" WITHOUT ANY WARRANTY OF ANY KIND. GENESYS
 * HEREBY DISCLAIMS ALL EXPRESS, IMPLIED, OR STATUTORY CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES WITH RESPECT TO THIS CODE (OR ANY
 * PART THEREOF), INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT. GENESYS AND ITS SUPPLIERS SHALL NOT BE LIABLE FOR
 * ANY DAMAGE SUFFERED AS A RESULT OF USING THIS CODE. IN NO EVENT
 * SHALL GENESYS AND ITS SUPPLIERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * CONSEQUENTIAL, ECONOMIC, INCIDENTAL, OR SPECIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, ANY LOST REVENUES OR PROFITS).
 *
 * Copyright (C) 2008 Genesys Telecommunications Laboratories,
 * Inc. All rights reserved.
 =====================================================================*/

package openMediaPlatformSDKExample;

import java.net.URI;
import java.net.URISyntaxException;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EtchedBorder;

import com.genesyslab.platform.commons.collections.KeyValueCollection;
import com.genesyslab.platform.commons.collections.KeyValuePair;
import com.genesyslab.platform.commons.protocol.Message;
import com.genesyslab.platform.commons.protocol.ProtocolException;

import com.genesyslab.platform.applicationblocks.commons.Action;
import com.genesyslab.platform.applicationblocks.commons.protocols.InteractionServerConfiguration;
import com.genesyslab.platform.applicationblocks.commons.protocols.ProtocolManagementServiceImpl;

import com.genesyslab.platform.openmedia.protocol.interactionserver.InteractionClient;
import com.genesyslab.platform.openmedia.protocol.interactionserver.InteractionProperties;
import com.genesyslab.platform.openmedia.protocol.interactionserver.events.EventAck;
import com.genesyslab.platform.openmedia.protocol.interactionserver.events.EventError;
import com.genesyslab.platform.openmedia.protocol.interactionserver.events.EventInvite;
import com.genesyslab.platform.openmedia.protocol.interactionserver.events.EventCurrentAgentStatus;
import com.genesyslab.platform.openmedia.protocol.interactionserver.requests.agentmanagement.RequestAgentLogin;
import com.genesyslab.platform.openmedia.protocol.interactionserver.requests.agentmanagement.RequestAgentLogout;
import com.genesyslab.platform.openmedia.protocol.interactionserver.requests.agentmanagement.RequestCancelNotReadyForMedia;
import com.genesyslab.platform.openmedia.protocol.interactionserver.requests.agentmanagement.RequestNotReadyForMedia;
import com.genesyslab.platform.openmedia.protocol.interactionserver.requests.interactiondelivery.RequestAccept;
import com.genesyslab.platform.openmedia.protocol.interactionserver.requests.interactionmanagement.RequestStopProcessing;

public class OpenMediaPlatformSDKExample {
	
	// Enter your Interaction Server information here:
	private String protocolName = "<protocol>";
	private String ixnServerHost = "<hostname>";
	private int ixnServerPort = <port>;
	private String mediaType = "<media type>";
	private String placeId = "<place name>";
	private long timeoutLength = 10000;
	// End of Interaction Server information.

	// Declare constants that will be used in our sample
    private JTextArea logTextArea = null;
    private JTextArea ixnTextArea = null;
	private JPanel contentPane = null;
	private JButton loginButton = null;
	private JButton logoutButton = null;
	private JButton readyButton = null;
	private JButton notReadyButton = null;
	private JButton receiveButton = null;
	private JButton acceptButton = null;
	private JButton doneButton = null;
	
	// Application block objects used in this sample
	private InteractionServerConfiguration ixnServerConfiguration;
	private ProtocolManagementServiceImpl mProtocolManager;
	
	private int incomingInteractionTicketId = 0;
	private String incomingInteractionName = null;
	private InteractionProperties incomingInteractionProperties = null;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		OpenMediaPlatformSDKExample openMediaExample = new OpenMediaPlatformSDKExample();
		// create and display the GUI
		openMediaExample.initializeGui();
		// initialize the Protocol Manager application block object
		openMediaExample.initializeProtocol();
	}

	private void initializeProtocol(){
		// create the configuration object used to connect to Stat Server
		ixnServerConfiguration = new InteractionServerConfiguration(protocolName);
    	ixnServerConfiguration.setClientName("default");
    	ixnServerConfiguration.setClientType(InteractionClient.AgentApplication);
    	try{
    		ixnServerConfiguration.setUri(new URI("tcp://" 
    				+ ixnServerHost + ":" + ixnServerPort));
    	}
    	catch(URISyntaxException e){
    		// add error handling
    		e.printStackTrace();
    	}
		appendLogMessage("InteractionServerProtocol object created.\n"
				+ "Connected to " + ixnServerConfiguration.getUri() + "\n\n");

    	// register the configuration object with Protocol Manager, and open the protocol  
    	mProtocolManager = new ProtocolManagementServiceImpl();
    	mProtocolManager.register(ixnServerConfiguration);
    	try{
    		mProtocolManager.getProtocol(protocolName).open();
    	}
    	catch(Exception e){
    		// add error handling
    		e.printStackTrace();
    	}
	}
	
    class OnEventError implements Action<Message> {
    	public void handle(Message obj) {
    		if (obj == null)
    			System.out.println("nothing returned");
    		else
    			// handle EventPackageError
    			appendLogMessage("=== Event: Package Error ===\n"
					+ obj.toString()
					+ "\n\n" + createTimeStamp() 
					+ "\n===========================");
    	}
    }
    
    private String createTimeStamp() {
        // Create a time stamp for application log window messages
        return new SimpleDateFormat("yyyy-dd-MM HH:mm:ss:SSS").format(new GregorianCalendar().getTime());
    }
    
	private boolean checkReturnMessage(Message respondingEvent, String requestSent)
	{
		if (respondingEvent == null) {
			System.out.println("Debug: Null message sent to checkReturnMessage function.");
			return false;
		}
		// Check to see if a request was acknowledged or not
		boolean success = false;
		String messageText;
		switch(respondingEvent.messageId())
		{
			case EventAck.ID:
				messageText = "  " + respondingEvent.messageName() + " returned - Request was successful.\n\n";
				success = true;
				break;
			case EventError.ID: 
				messageText = respondingEvent.messageName() + " failed with an error: \n"
					+ respondingEvent.toString() + "\n";
				break;
			default:
				messageText = "Unexpected Event: " + respondingEvent.messageName() + "\n\n"
					+ respondingEvent.toString() + "\n";
				break;
		}
		appendLogMessage(requestSent + messageText);
		return success;
	}
    
    private void logicLogin(){
    	try {
    		RequestAgentLogin reqAgentLogin = RequestAgentLogin.create(101, placeId, null);
    		KeyValueCollection kvc = new KeyValueCollection();
    		kvc.addPair(new KeyValuePair(mediaType, mediaType));
    		reqAgentLogin.setMediaList(kvc);
    		Message msgResponse = mProtocolManager.getProtocol(protocolName).request(reqAgentLogin);
    		if (checkReturnMessage(msgResponse, "RequestAgentLogin sent:\n")){
				logoutButton.setEnabled(true);
				readyButton.setEnabled(false);
				readyButton.setEnabled(true);
				receiveButton.setEnabled(true);    			
    		}
    		else {
    			loginButton.setEnabled(true);
    			appendLogMessage("Could not login. Null message returned on RequestAgentLogin attempt.\n\n");
    		}
    	}
    	catch (ProtocolException e) {
    		e.printStackTrace();
    	}
    }
    
    private void logicLogout(){
    	try {
    		RequestAgentLogout reqAgentLogout = RequestAgentLogout.create();
    		Message msgResponse = mProtocolManager.getProtocol(protocolName).request(reqAgentLogout);
    		if (checkReturnMessage(msgResponse, "RequestAgentLogout sent:\n")){
				loginButton.setEnabled(true);
				readyButton.setEnabled(false);
				notReadyButton.setEnabled(false);
				receiveButton.setEnabled(false);
				acceptButton.setEnabled(false);
				doneButton.setEnabled(false);
				ixnTextArea.setText("");
    		}
    		else
    			loginButton.setEnabled(true);
    	}
    	catch (ProtocolException e) {
    		e.printStackTrace();
    	}
    }
    
    private void logicReady(){
		RequestCancelNotReadyForMedia requestNotReady = RequestCancelNotReadyForMedia.create(mediaType, null);
		String strRequestType = "RequestNotReadyForMedia sent:\n";
		try {
			Message response = mProtocolManager.getProtocol(protocolName).request(requestNotReady);
			if (checkReturnMessage(response, strRequestType)) {
				notReadyButton.setEnabled(true);
				receiveButton.setEnabled(true);
			}
			else {
				readyButton.setEnabled(true);
			}
		}
		catch (ProtocolException protocolException) {
			appendLogMessage("Protocol Exception!\n" + protocolException);
		}
    }
    
    private void logicNotReady(){
		RequestNotReadyForMedia requestReady = RequestNotReadyForMedia.create(mediaType, null);
		String strRequestType = "RequestCancelNotReadyForMedia sent:\n";
		try {
			Message response = mProtocolManager.getProtocol(protocolName).request(requestReady);
			if (checkReturnMessage(response, strRequestType)) {
				readyButton.setEnabled(true);
				acceptButton.setEnabled(false);
				doneButton.setEnabled(false);
				ixnTextArea.setText("");
			}
			else {
				notReadyButton.setEnabled(true);
			}
		}
		catch (ProtocolException protocolException) {
			appendLogMessage("Protocol Exception!\n" + protocolException);
		}
    }
    
    private void logicReceive(){
		try {
			Message unsolicitedEvent = mProtocolManager.getReceiver().receive(timeoutLength);
			if (unsolicitedEvent == null) {
				appendLogMessage("No message received. The queue is empty!\n\n");
				receiveButton.setEnabled(true);
			}
			else
			{
				String logMessage = "";
				switch(unsolicitedEvent.messageId())
				{
					case EventInvite.ID: // We are invited...
						EventInvite eventInvite = (EventInvite) unsolicitedEvent;
						// save there parameters so that we can accept the invitation
						incomingInteractionTicketId = eventInvite.getTicketId();
						incomingInteractionProperties = eventInvite.getInteraction();
						incomingInteractionName = eventInvite.getInteraction().getInteractionId();
						// show details of the interaction message
						logMessage = "Interaction ID: " + incomingInteractionName 
							+ "\nQueue: " + eventInvite.getInteraction().getInteractionQueue()
							+ "\nInteraction Subtype: " + eventInvite.getInteraction().getInteractionSubtype()
							+ "\nSubject: " + eventInvite.getInteraction().getInteractionUserData().getString("Subject")
							+ "\nReceived At: " + eventInvite.getInteraction().getInteractionReceivedAt();
						ixnTextArea.setText(logMessage);
						// display message
						logMessage = "EventInvite received.\n\n"
							+ "eventInvite.TicketId: " + incomingInteractionTicketId + "\n" 
							+ "eventInvite.Interaction.InteractionId: " 
							+ incomingInteractionName + "\n\n";
						acceptButton.setEnabled(true);
						break;
					case EventCurrentAgentStatus.ID:
						logMessage = "EventCurrentAgentStatus event received.\n\n";
						receiveButton.setEnabled(true);
						break;
					case EventError.ID: // 126
						logMessage = "EventError received. Details follow:\n"
							+ unsolicitedEvent.toString() + "\n\n";
						receiveButton.setEnabled(true);
						break;
					default:
						logMessage = "Unexpected Event: " + unsolicitedEvent.messageName() + "\n"
							+ "Responding Event ID is " + unsolicitedEvent.messageId() + "\n"
							+ unsolicitedEvent.toString() + "\n\n";
						receiveButton.setEnabled(true);
						break;
				}
				appendLogMessage(logMessage);
			}
		}
		catch (InterruptedException e) {
			e.printStackTrace();
		}
    	
    }
    
    private void logicAccept(){
		try {
			RequestAccept requestAccept = 
				RequestAccept.create(incomingInteractionTicketId, incomingInteractionName);
			Message respondingEvent = mProtocolManager.getProtocol(protocolName).request(requestAccept);
			if (checkReturnMessage(respondingEvent, "RequestAccept sent:\n")) {
				appendLogMessage("Interaction ID: " + requestAccept.getInteractionId() + "\n"
					+ "Queue: " + incomingInteractionProperties.getInteractionQueue() + "\n"
					+ "Interaction Subtype: " + incomingInteractionProperties.getInteractionSubtype() + "\n\n"
					+ "User Data: \n" + incomingInteractionProperties.getInteractionUserData().toString() + "\n\n"
					+ "Received At: " + incomingInteractionProperties.getInteractionReceivedAt().toString() + "\n\n");
				doneButton.setEnabled(true);
			}
			else {
				acceptButton.setEnabled(true);
				receiveButton.setEnabled(true);
			}
		}
		catch (ProtocolException protocolException) {
			appendLogMessage("Protocol Exception!\n" + protocolException);
		}
    }
    
    private void logicDone(){
    	try {
    		RequestStopProcessing reqDone = RequestStopProcessing.create(incomingInteractionProperties.getInteractionId(), null);
    		Message msgResponse = mProtocolManager.getProtocol(protocolName).request(reqDone);
    		if (checkReturnMessage(msgResponse, "RequestStopProcessing sent:\n")){
    			ixnTextArea.setText("");
    			receiveButton.setEnabled(true);
    		}
    		else
    			doneButton.setEnabled(true);
    	}
    	catch (ProtocolException e){
    		e.printStackTrace();
    	}
    }

	
//=================================================
//					GUI Code    
//=================================================
	
	/**
     * This method initializes mainGuiWindow
     *
     * @return javax.swing.JFrame
     */
    private void initializeGui() {
        JFrame mainGuiWindow = new JFrame();
        mainGuiWindow.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        mainGuiWindow.setTitle("Open Media Client Platform SDK Example");
        mainGuiWindow.setSize(new Dimension(640, 570));
        mainGuiWindow.setMinimumSize(new Dimension(640, 570));
        mainGuiWindow.setContentPane(getContentPane());
        mainGuiWindow.setVisible(true);
    }

    /**
     * This method initializes contentPane
     *
     * @return javax.swing.JPanel
     */
    private JPanel getContentPane() {
    	//declare variables used to create GUI
    	contentPane = new JPanel(new GridBagLayout());

    	//add a new panel to display status information
    	JPanel mPanel = newPanel("Controls");
    	GridBagConstraints mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.NONE;
    	mGridBagConstraints.anchor = GridBagConstraints.NORTH;
    	mGridBagConstraints.gridx = 0;
    	mGridBagConstraints.gridy = 0;
    	contentPane.add(mPanel, mGridBagConstraints);

    	//add the first (Login) button
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.NONE;
    	mGridBagConstraints.gridx = 0;
    	mGridBagConstraints.gridy = 0;
    	loginButton = newButton(125, 25, "Login");
    	loginButton.setEnabled(true);
    	loginButton.addActionListener(
    			// Create the ActionListener object for the Login button
    			new ActionListener(){
    				public void actionPerformed(ActionEvent e){
    					// application logic for the Login button
    					loginButton.setEnabled(false);
    					// login action
    					logicLogin();
    				}
    			});
    	mPanel.add(loginButton, mGridBagConstraints);        

    	//add the second (Logout) button
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.NONE;
    	mGridBagConstraints.gridx = 1;
    	mGridBagConstraints.gridy = 0;
    	mGridBagConstraints.weighty = 1.0;
    	logoutButton = newButton(125, 25, "Logout");
    	logoutButton.setEnabled(false);
    	logoutButton.addActionListener(
    			// Create the ActionListener object for the Logout button
    			new ActionListener(){
    				public void actionPerformed(ActionEvent e){
    					// application logic for the Logout button
    					logoutButton.setEnabled(false);
    					// logout action
    					logicLogout();
    				}
    			});
    	mPanel.add(logoutButton, mGridBagConstraints);

    	//add the third (Ready) button
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.NONE;
    	mGridBagConstraints.gridx = 0;
    	mGridBagConstraints.gridy = 2;
    	readyButton = newButton(125, 25, "Ready");
    	readyButton.setEnabled(false);
    	readyButton.addActionListener(
    			// Create the ActionListener object for the Ready button
    			new ActionListener(){
    				public void actionPerformed(ActionEvent e){
    					// application logic for the Ready button
    					readyButton.setEnabled(false);
    					// ready action
    					logicReady();
    				}
    			});
    	mPanel.add(readyButton, mGridBagConstraints);

    	//add the fourth (Not Ready) button
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.NONE;
    	mGridBagConstraints.gridx = 1;
    	mGridBagConstraints.gridy = 2;
    	notReadyButton = newButton(125, 25, "Not Ready");
    	notReadyButton.setEnabled(false);
    	notReadyButton.addActionListener(
    			// Create the ActionListener object for the Not Ready button
    			new ActionListener(){
    				public void actionPerformed(ActionEvent e){
    					// application logic for the Not Ready button
    					notReadyButton.setEnabled(false);
    					// not ready action
    					logicNotReady();
    				}
    			});
    	mPanel.add(notReadyButton, mGridBagConstraints);      

    	// add the fifth (Receive) button
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.NONE;
    	mGridBagConstraints.gridx = 0;
    	mGridBagConstraints.gridy = 4;
    	receiveButton = newButton(125, 25, "Receive");
    	receiveButton.setEnabled(false);
    	receiveButton.addActionListener(
    			// Create the ActionListener object for the Receive button
    			new ActionListener(){
    				public void actionPerformed(ActionEvent e){
    					// application logic for the Receive button
    					receiveButton.setEnabled(false);
    					// receive action
    					logicReceive();
    				}
    			});
    	mPanel.add(receiveButton, mGridBagConstraints);

    	// add the sixth (Accept) button
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.NONE;
    	mGridBagConstraints.gridx = 1;
    	mGridBagConstraints.gridy = 4;
    	acceptButton = newButton(125, 25, "Accept");
    	acceptButton.setEnabled(false);
    	acceptButton.addActionListener(
    			// Create the ActionListener object for the Accept button
    			new ActionListener(){
    				public void actionPerformed(ActionEvent e){
    					// application logic for the Accept button
    					acceptButton.setEnabled(false);
    					receiveButton.setEnabled(false);
    					// accept action
    					logicAccept();
    				}
    			});
    	mPanel.add(acceptButton, mGridBagConstraints);

    	// add the seventh (Done) button
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.gridx = 1;
    	mGridBagConstraints.gridy = 6;
    	doneButton = newButton(125, 25, "Done");
    	doneButton.setEnabled(false);
    	doneButton.addActionListener(
    			// Create the ActionListener object for the Done button
    			new ActionListener(){
    				public void actionPerformed(ActionEvent e){
    					// application logic for the Done button
    					doneButton.setEnabled(false);
    					// done action
    					logicDone();
    				}
    			});
    	mPanel.add(doneButton, mGridBagConstraints);

    	//add a new panel to display status information
    	mPanel = newPanel("Interaction Details");
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
    	mGridBagConstraints.gridx = 1;
    	mGridBagConstraints.gridy = 0;
    	contentPane.add(mPanel, mGridBagConstraints);

	  	//add a text pane to the panel
	  	mGridBagConstraints = new GridBagConstraints();
	  	mGridBagConstraints.fill = GridBagConstraints.BOTH;
	  	mGridBagConstraints.gridx = 0;
	  	mGridBagConstraints.gridy = 0;
	  	ixnTextArea = new JTextArea(7, 20);
	  	ixnTextArea.setEditable(false);
    	JScrollPane mScrollPane = new JScrollPane(ixnTextArea);
        mScrollPane.setPreferredSize(new Dimension(300, 200));
        mScrollPane.setAutoscrolls(true);
        mScrollPane.setBackground(new Color(255, 255, 227));
	  	mPanel.add(mScrollPane, mGridBagConstraints);

    	//add a text pane to the panel
    	mGridBagConstraints = new GridBagConstraints();
    	mGridBagConstraints.fill = GridBagConstraints.BOTH;
    	mGridBagConstraints.gridx = 0;
    	mGridBagConstraints.gridy = 1;
    	mGridBagConstraints.gridwidth = 2;
    	logTextArea = new JTextArea(7, 20);
    	logTextArea.setEditable(false);
    	mScrollPane = new JScrollPane(logTextArea);
        mScrollPane.setPreferredSize(new Dimension(600, 300));
        mScrollPane.setAutoscrolls(true);
        mScrollPane.setBackground(new Color(255, 255, 227));
	  	mPanel.add(mScrollPane, mGridBagConstraints);
    	contentPane.add(mScrollPane, mGridBagConstraints);
    	//return the JPanel object we've created
    	return contentPane;
    }

    private JPanel newPanel(String jpName){
    	JPanel mPanel = new JPanel(new GridBagLayout());
    	//set up the panel
    	mPanel.setBorder(BorderFactory.createTitledBorder(
    			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), jpName));
    	return mPanel;
    }
    
    private JButton newButton(int jbWidth, int jbHeight, String jbText){
    	JButton mButton = new JButton();
    	mButton.setPreferredSize(new Dimension(jbWidth, jbHeight));
    	mButton.setText(jbText);
    	return mButton;
    }
    
    private void appendLogMessage(String eventString){
    	try {
    		logTextArea.append(eventString);
    		logTextArea.setCaretPosition(logTextArea.getText().length()-1);
    	}
    	catch (Exception e){
    		System.err.println("Couldn't insert text into text area."
    				+ "\n" + e.toString());
    	}
    }

}
